package com.qmth.paper.library.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.paper.library.business.bean.result.PaperArchivesDetailResult;
import com.qmth.paper.library.business.bean.result.PaperArchivesResult;
import com.qmth.paper.library.business.bean.result.SelectResult;
import com.qmth.paper.library.business.entity.PaperArchives;
import com.qmth.paper.library.business.entity.PaperLibrary;
import com.qmth.paper.library.business.entity.PaperScanTask;
import com.qmth.paper.library.business.mapper.PaperArchivesMapper;
import com.qmth.paper.library.business.service.PaperArchivesService;
import com.qmth.paper.library.business.service.PaperLibraryService;
import com.qmth.paper.library.business.service.PaperScanTaskService;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.util.ServletUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 档案管理表 服务实现类
 * </p>
 */
@Service
public class PaperArchivesServiceImpl extends ServiceImpl<PaperArchivesMapper, PaperArchives> implements PaperArchivesService {

    @Resource
    PaperScanTaskService paperScanTaskService;

    @Resource
    PaperLibraryService paperLibraryService;

    @Override
    public IPage<PaperArchivesResult> pageData(Long semesterId, Long managerOrgId, Long archivesTypeId, String archivesName, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return this.baseMapper.pageData(new Page<>(pageNumber, pageSize), schoolId, semesterId, managerOrgId, archivesTypeId, archivesName);
    }

    @Override
    public boolean saveData(PaperArchives paperArchives) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        // 新增
        if (paperArchives.getId() == null) {
            PaperArchives archives = this.baseMapper.selectByArchivesName(schoolId, paperArchives.getArchivesName());
            if (archives != null) {
                throw ExceptionResultEnum.ERROR.exception("档案名称已存在");
            }
            paperArchives.setId(SystemConstant.getDbUuid());
            paperArchives.setSchoolId(schoolId);
            // 档案编号（时间戳）
            paperArchives.setArchivesCode(String.valueOf(System.currentTimeMillis()));
            paperArchives.setCreateId(sysUser.getId());
            paperArchives.setCreateTime(System.currentTimeMillis());
            return this.save(paperArchives);
        } else {
            PaperArchives archives = this.baseMapper.selectByArchivesName(schoolId, paperArchives.getArchivesName());
            if (archives != null && !archives.getId().equals(paperArchives.getId())) {
                throw ExceptionResultEnum.ERROR.exception("档案名称已存在");
            }
            PaperArchives oldPaperArchives = this.getById(paperArchives.getId());
            // 档案编号（时间戳）
            oldPaperArchives.setArchivesName(paperArchives.getArchivesName());
            oldPaperArchives.setArchivesTypeId(paperArchives.getArchivesTypeId());
            oldPaperArchives.setSemesterId(paperArchives.getSemesterId());
            oldPaperArchives.setManagerId(paperArchives.getManagerId());
            oldPaperArchives.setManagerOrgId(paperArchives.getManagerOrgId());
            oldPaperArchives.setWarningTime(paperArchives.getWarningTime());
            paperArchives.setUpdateId(sysUser.getId());
            paperArchives.setUpdateTime(System.currentTimeMillis());
            return this.updateById(paperArchives);
        }
    }

    @Override
    public boolean delete(Long id) {
        PaperArchives paperArchives = this.getById(id);
        if (paperArchives == null) {
            throw ExceptionResultEnum.ERROR.exception("数据有误");
        }

        List<PaperScanTask> paperScanTaskList = paperScanTaskService.listByPaperArchivesId(id);
        if (!paperScanTaskList.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("已有扫描任务，不能删除");
        }
        return this.removeById(id);
    }

    @Override
    public List<SelectResult> queryManagerOrg(Long semesterId) {
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));
        return this.baseMapper.queryManagerOrg(schoolId, semesterId);
    }

    @Override
    public List<SelectResult> queryArchives(Long semesterId) {
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));
        QueryWrapper<PaperArchives> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PaperArchives::getSchoolId, schoolId);
        if (semesterId != null) {
            queryWrapper.lambda().eq(PaperArchives::getSemesterId, semesterId);
        }
        List<PaperArchives> paperArchivesList = this.list(queryWrapper);
        return paperArchivesList.stream().map(m -> {
            SelectResult result = new SelectResult();
            result.setId(String.valueOf(m.getId()));
            result.setCode(m.getArchivesCode());
            result.setName(m.getArchivesName());
            return result;
        }).collect(Collectors.toList());
    }

    @Override
    public IPage<PaperArchivesDetailResult> pageDetail(Long paperArchivesId, Long belongOrgId, String majorName, String clazzName, String courseName, String teacherName, String teachClazzName, Boolean isBind, String param, Integer pageNumber, Integer pageSize) {
        IPage<PaperArchivesDetailResult> detailResultIPage = this.baseMapper.pageDetail(new Page<>(pageNumber, pageSize), paperArchivesId, belongOrgId, majorName, clazzName, courseName, teacherName, teachClazzName, isBind, param);
        for (PaperArchivesDetailResult record : detailResultIPage.getRecords()) {
            QueryWrapper<PaperLibrary> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(PaperLibrary::getPaperScanTaskDetailId, record.getPaperScanTaskDetailId());
            int count = paperLibraryService.count(queryWrapper);
            record.setBindCount(count);
        }
        return detailResultIPage;
    }

}
