package com.qmth.paper.library.business.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.paper.library.business.bean.result.PaperLibraryResult;
import com.qmth.paper.library.business.bean.result.TaskStudentResult;
import com.qmth.paper.library.business.entity.PaperLibrary;
import com.qmth.paper.library.business.entity.PaperScanTaskDetail;
import com.qmth.paper.library.business.mapper.PaperLibraryMapper;
import com.qmth.paper.library.business.service.PaperLibraryCommonService;
import com.qmth.paper.library.business.service.PaperLibraryService;
import com.qmth.paper.library.business.service.PaperScanTaskDetailService;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.lock.LockService;
import com.qmth.paper.library.common.lock.LockType;
import com.qmth.paper.library.common.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 图片库 服务实现类
 * </p>
 */
@Service
public class PaperLibraryServiceImpl extends ServiceImpl<PaperLibraryMapper, PaperLibrary> implements PaperLibraryService {

    @Resource
    PaperScanTaskDetailService paperScanTaskDetailService;

    @Resource
    PaperLibraryCommonService paperLibraryCommonService;

    @Resource
    LockService lockService;

    @Override
    public IPage<PaperLibraryResult> pageUnbindData(Long paperScanTaskId, Integer pageNumber, Integer pageSize) {
        return this.baseMapper.pageUnbindData(new Page<>(pageNumber, pageSize), paperScanTaskId);
    }

    @Override
    public int countUnbindData(Long paperScanTaskId) {
        return this.baseMapper.countUnbindData(paperScanTaskId);
    }

    @Override
    public IPage<PaperLibraryResult> pageBindData(Long paperScanTaskId, Integer pageNumber, Integer pageSize) {
        return this.baseMapper.pageBindData(new Page<>(pageNumber, pageSize), paperScanTaskId);
    }

    @Override
    public int countBindData(Long paperScanTaskId) {
        return this.baseMapper.countBindData(paperScanTaskId);
    }

    @Transactional
    @Override
    public boolean bind(Long paperLibraryId, Long paperScanTaskDetailId) {
        PaperScanTaskDetail paperScanTaskDetail = paperScanTaskDetailService.getById(paperScanTaskDetailId);
        if (paperScanTaskDetail == null) {
            throw ExceptionResultEnum.ERROR.exception("绑定对象有误，任务下无该考生，请刷新数据再试");
        }
        PaperLibrary paperLibrary = this.getById(paperLibraryId);
        if (paperLibrary == null) {
            throw ExceptionResultEnum.ERROR.exception("绑定对象有误，图片已删除，请刷新数据再试");
        }

        // 更新已绑定状态
        if (!paperScanTaskDetail.getBind()) {
            paperScanTaskDetail.setBind(true);
            paperScanTaskDetailService.updateById(paperScanTaskDetail);
        }

        UpdateWrapper<PaperLibrary> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(PaperLibrary::getPaperScanTaskDetailId, paperScanTaskDetailId)
                .eq(PaperLibrary::getId, paperLibraryId);
        return this.update(updateWrapper);
    }

    @Override
    public List<String> listStudentPicture(Long paperScanTaskDetailId) {
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));
        QueryWrapper<PaperLibrary> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PaperLibrary::getSchoolId, schoolId)
                .eq(PaperLibrary::getPaperScanTaskDetailId, paperScanTaskDetailId)
                .orderByAsc(PaperLibrary::getSequence);
        List<PaperLibrary> paperLibraryList = this.list(queryWrapper);
        List<String> fileUrls = new ArrayList<>();
        for (PaperLibrary paperLibrary : paperLibraryList) {
            String fileUrl = paperLibrary.getPath();
            if (StringUtils.isBlank(fileUrl)) {
                throw ExceptionResultEnum.ERROR.exception("未找到图片路径");
            }
            List<JSONObject> objectList = JSON.parseArray(fileUrl, JSONObject.class);
            for (JSONObject jsonObject : objectList) {
                fileUrls.add(paperLibraryCommonService.preViewPath(jsonObject.getString(SystemConstant.PATH), jsonObject.getString(SystemConstant.UPLOAD_TYPE), false));
            }
        }
        return fileUrls;
    }

    @Override
    public List<PaperLibraryResult> toBindPaper() {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        Long schoolId = SystemConstant.convertIdToLong(String.valueOf(ServletUtil.getRequestHeaderSchoolId()));

        // 查询下一个待绑定任务
        List<PaperLibrary> waitPaperLibraryList = this.baseMapper.selectBatchData(schoolId, sysUser.getId());
        if (waitPaperLibraryList.isEmpty()) {
            waitPaperLibraryList = createBindData(sysUser.getId(), schoolId);
        }

        List<PaperLibraryResult> paperLibraryResultList = new ArrayList<>();
        if (!waitPaperLibraryList.isEmpty()) {
            for (PaperLibrary paperLibrary : waitPaperLibraryList) {
                PaperLibraryResult paperLibraryResult = new PaperLibraryResult();
                BeanUtils.copyProperties(paperLibrary, paperLibraryResult);
                String fileUrl = paperLibrary.getPath();
                if (StringUtils.isBlank(fileUrl)) {
                    throw ExceptionResultEnum.ERROR.exception("未找到图片路径");
                }
                List<JSONObject> objectList = JSON.parseArray(fileUrl, JSONObject.class);
                List<String> fileUrls = new ArrayList<>();
                for (JSONObject jsonObject : objectList) {
                    fileUrls.add(paperLibraryCommonService.preViewPath(jsonObject.getString(SystemConstant.PATH), jsonObject.getString(SystemConstant.UPLOAD_TYPE), false));
                }
                paperLibraryResult.setFileUrls(fileUrls);
                paperLibraryResultList.add(paperLibraryResult);
            }
        }
        return paperLibraryResultList;
    }

    @Override
    public List<TaskStudentResult> listByStudentCode(String studentCode) {
        if(StringUtils.isBlank(studentCode)){
            throw ExceptionResultEnum.ERROR.exception("请输入考号");
        }
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return this.baseMapper.listByStudentCode(schoolId, studentCode);
    }

    private List<PaperLibrary> createBindData(Long userId, Long schoolId) {
        try {
            lockService.waitlock(LockType.BIND_PAPER_TASK, schoolId);
            List<PaperLibrary> paperLibraryList = this.baseMapper.listUnBindData(schoolId);
            if (!paperLibraryList.isEmpty()) {
                paperLibraryList.forEach(m -> m.setUserId(userId));
                this.updateBatchById(paperLibraryList);
            }
            return paperLibraryList;
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            lockService.unlock(LockType.BIND_PAPER_TASK, schoolId);
        }
        return new ArrayList<>();
    }

}
