package com.qmth.paper.library.api;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.paper.library.common.bean.params.UserSaveParams;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.service.SysUserService;
import com.qmth.paper.library.common.util.Result;
import com.qmth.paper.library.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.security.NoSuchAlgorithmException;

/**
 * <p>
 * 用户表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "用户Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.sys}/user")
@Validated
//@Aac(strict = BOOL.FALSE, auth = BOOL.FALSE)
public class SysUserController {
    private final static Logger log = LoggerFactory.getLogger(SysUserController.class);

    @Resource
    private SysUserService sysUserService;

    /**
     * 用户列表查询
     *
     * @param userInfo   用户信息（账号/姓名/手机号）
     * @param roleId
     * @param orgId
     * @param enable
     * @param pageNumber
     * @param pageSize
     */
    @ApiOperation(value = "查询")
    @PostMapping("/list")
    public Result list(@RequestParam(value = "userInfo", required = false) String userInfo,
                       @RequestParam(value = "orgId", required = false) String orgId,
                       @RequestParam(value = "roleId", required = false) String roleId,
                       @RequestParam(value = "enable", required = false) Boolean enable,
                       @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(sysUserService.listForPrintSystem(userInfo, SystemConstant.convertIdToLong(orgId), roleId, enable, pageNumber, pageSize));
    }

    /**
     * 新增/修改
     *
     * @param userSaveParams
     */
    @ApiOperation(value = "新增/修改")
    @PostMapping("/save")
    public Result save(@ApiParam(value = "用户信息", required = true) @Valid @RequestBody UserSaveParams userSaveParams, BindingResult bindingResult) throws IllegalAccessException {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }
        return ResultUtil.ok(sysUserService.saveUser(userSaveParams));
    }

    /**
     * 启用/禁用
     *
     * @param user
     */
    @ApiOperation(value = "启用/禁用")
    @PostMapping("/enable")
    public Result enable(@RequestBody SysUser user) throws NoSuchAlgorithmException, IllegalAccessException {
        return ResultUtil.ok(sysUserService.enable(user));
    }

    /**
     * 重置密码
     *
     * @param user
     */
    @ApiOperation(value = "重置密码")
    @PostMapping("/reset_password")
    public Result resetPassword(@RequestBody SysUser user) throws NoSuchAlgorithmException {
        return ResultUtil.ok(sysUserService.resetPassword(user.getId()));
    }

    /**
     * 修改密码
     */
    @ApiOperation(value = "修改密码")
    @PostMapping("/update_password")
    @Aac(strict = BOOL.FALSE, auth = BOOL.FALSE)
    public Result updatePassword(@RequestBody SysUser user) throws NoSuchAlgorithmException {
        return ResultUtil.ok(sysUserService.updatePassword(user));
    }

    /**
     * 绑定角色
     *
     * @param sysUser
     */
    @ApiOperation(value = "绑定角色")
    @PostMapping("/bind_roles")
    public Result bindRoles(@RequestBody SysUser sysUser) {
        return ResultUtil.ok(sysUserService.bindRoles(sysUser));
    }

}

